﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace AnimatedDottedOutlinedTextSansFlicker
{
    public partial class MainPage : UserControl
    {
        const double STROKE_THICKNESS = 6;
        const double DOT_SIZE = 0.1;
        const double AVG_DASH_SPACE = 1.5;

        public MainPage()
        {
            InitializeComponent();
            Brush blueBrush = new SolidColorBrush(Colors.Blue);
            Storyboard storyboard = new Storyboard();

            while (pathFigureCollection.Count > 0)
            {
                // Remove PathFigure from resource collection so it
                //  can be added to the PathGeometry Figures collection.
                PathFigure pathFigure = pathFigureCollection[0];
                pathFigureCollection.RemoveAt(0);

                // Calculate optimum dot size
                double length = GetPathFigureLength(pathFigure);
                int numberDots = (int)Math.Round(length / (STROKE_THICKNESS * (AVG_DASH_SPACE + DOT_SIZE)));
                double dashSpace = length / (numberDots * STROKE_THICKNESS) - DOT_SIZE;

                // Create DoubleCollection for StrokeDashArray property
                DoubleCollection strokeDashArray = new DoubleCollection();
                strokeDashArray.Add(DOT_SIZE);
                strokeDashArray.Add(dashSpace);

                // Create the PathGeometry
                PathGeometry pathGeometry = new PathGeometry();
                pathGeometry.Figures.Add(pathFigure);

                // Create the Path
                Path path = new Path()
                {
                    Stroke = blueBrush,
                    StrokeThickness = STROKE_THICKNESS,
                    StrokeDashCap = PenLineCap.Round,
                    StrokeDashArray = strokeDashArray,
                    Data = pathGeometry
                };

                // Create the animation targetting the StrokeDashOffset property
                DoubleAnimation anima = new DoubleAnimation()
                {
                    From = dashSpace + DOT_SIZE,
                    To = 0,
                    Duration = new Duration(TimeSpan.FromSeconds(1)),
                    RepeatBehavior = RepeatBehavior.Forever
                };

                // Final per-figure setup
                Storyboard.SetTarget(anima, path);
                Storyboard.SetTargetProperty(anima, new PropertyPath(Path.StrokeDashOffsetProperty));
                storyboard.Children.Add(anima);
                LayoutRoot.Children.Add(path);
            }
            // Begin the animation
            storyboard.Begin();
        }

        double GetPathFigureLength(PathFigure pathFigure)
        {
            Point pt0 = pathFigure.StartPoint;
            double length = 0;

            foreach (PathSegment pathSegment in pathFigure.Segments)
            {
                length += GetPathSegmentLength(pathSegment, ref pt0);
            }

            return length;
        }

        double GetPathSegmentLength(PathSegment pathSegment, ref Point pt0)
        {
            double length = 0;

            if (pathSegment is LineSegment)
            {
                LineSegment lineSegment = pathSegment as LineSegment;
                length += GetLineLength(pt0, lineSegment.Point);
                pt0 = lineSegment.Point;
            }
            else if (pathSegment is PolyLineSegment)
            {
                PolyLineSegment polyLineSegment = pathSegment as PolyLineSegment;

                foreach (Point pt in polyLineSegment.Points)
                {
                    length += GetLineLength(pt0, pt);
                    pt0 = pt;
                }
            }
            else if (pathSegment is BezierSegment)
            {
                BezierSegment bezierSegment = pathSegment as BezierSegment;
                length += GetBezierLength(pt0, bezierSegment.Point1, bezierSegment.Point2, bezierSegment.Point3);
                pt0 = bezierSegment.Point3;
            }
            else if (pathSegment is PolyBezierSegment)
            {
                PolyBezierSegment polyBezierSegment = pathSegment as PolyBezierSegment;
                PointCollection pts = polyBezierSegment.Points;

                for (int i = 0; i < pts.Count; i += 3)
                {
                    length += GetBezierLength(pt0, pts[i], pts[i + 1], pts[i + 2]);
                    pt0 = pts[i + 2];
                }
            }
            else
                throw new NotSupportedException("PathSegment of type " + pathSegment.GetType().Name + " is not supported");

            return length;
        }

        double GetBezierLength(Point pt0, Point pt1, Point pt2, Point pt3)
        {
            double length = 0;
            int count = (int)(GetLineLength(pt0, pt1) + GetLineLength(pt1, pt2) + GetLineLength(pt2, pt3));

            for (int i = 1; i <= count; i++)
            {
                double t = (double)i / count;
                double x = (1 - t) * (1 - t) * (1 - t) * pt0.X + 3 * t * (1 - t) * (1 - t) * pt1.X + 3 * t * t * (1 - t) * pt2.X + t * t * t * pt3.X;
                double y = (1 - t) * (1 - t) * (1 - t) * pt0.Y + 3 * t * (1 - t) * (1 - t) * pt1.Y + 3 * t * t * (1 - t) * pt2.Y + t * t * t * pt3.Y;
                Point pt = new Point(x, y);
                length += GetLineLength(pt0, pt);
                pt0 = pt;
            }
            return length;
        }

        double GetLineLength(Point pt1, Point pt2)
        {
            return Math.Sqrt(Math.Pow(pt1.X - pt2.X, 2) + Math.Pow(pt1.Y - pt2.Y, 2));
        }
    }
}
